@@ -67,7 +67,7 @@ def sndrcv(pks, pkt, timeout = None, inter = 0, verbose=None, chainCC=0, retry=0
     while retry >= 0:
         found=0
     
-        if timeout < 0:
+        if timeout is not None and timeout < 0:
             timeout = None
             
         rdpipe,wrpipe = os.pipe()
@@ -541,7 +541,7 @@ iface:    listen answers only on the given interface"""
 
 @conf.commands.register
 def sniff(count=0, store=1, offline=None, prn = None, lfilter=None, L2socket=None, timeout=None,
-          opened_socket=None, stop_filter=None, *arg, **karg):
+          opened_socket=None, stop_filter=None, exceptions=False, *arg, **karg):
     """Sniff packets
 sniff([count=0,] [prn=None,] [store=1,] [offline=None,] [lfilter=None,] + L2ListenSocket args) -> list of packets
 
@@ -560,6 +560,8 @@ opened_socket: provide an object ready to use .recv() on
 stop_filter: python function applied to each packet to determine
              if we have to stop the capture after this packet
              ex: stop_filter = lambda x: x.haslayer(TCP)
+exceptions: reraise caught exceptions such as KeyboardInterrupt
+            when a user interrupts sniffing
     """
     c = 0
     
@@ -602,9 +604,14 @@ stop_filter: python function applied to each packet to determine
                 if count > 0 and c >= count:
                     break
     except KeyboardInterrupt:
-        pass
-    if opened_socket is None:
-        s.close()
+        if exceptions:
+            raise
+        else:
+            pass
+    finally:
+        if opened_socket is None:
+            s.close()
+
     return plist.PacketList(lst,"Sniffed")
 
 
